#include <dbsearch.h>
#include "../foosion_helpers/helpers.h"

#include <history.h>

//////////////////////////////////////////////////
// This file implements support for foo_dbsearch.
// You will need the foo_dbsearch_mini_sdk to compile it.
// If you don't need foo_dbsearch support, simply don't compile this file.
//////////////////////////////////////////////////

class snapshot_history : public dbsearch::snapshot
{
	history_handle_list data;
	int now_playing, bound_prev, bound_next;
public:
	snapshot_history();
	virtual ~snapshot_history();
	virtual unsigned get_count();
	virtual metadb_handle * get_item(unsigned index);
	virtual void format_title(unsigned index, string_base & out, const char * spec, const char * extra_items);
	virtual bool is_ordered();
};

class domain_history : public dbsearch::domain
{
public:
	static const GUID domain_guid;
	virtual const GUID & get_domain_guid() {return domain_guid;}
	virtual const char * get_name();
	virtual dbsearch::snapshot * get_snapshot();
	virtual bool is_ordered();
};

//////////////////////////////////////////////////
// snapshot
//////////////////////////////////////////////////

snapshot_history::snapshot_history()
{
	history * p_history = history::get();
	p_history->history_lock();

	p_history->get_data(data);
	now_playing = p_history->get_now_playing();
	bound_prev = p_history->get_index_from_offset(-1) + 1;
	bound_next = p_history->get_index_from_offset(+1) - 1;

	p_history->history_unlock();
}

snapshot_history::~snapshot_history()
{
	data.delete_all();
}

unsigned snapshot_history::get_count()
{
	return data.get_count();
}

metadb_handle * snapshot_history::get_item(unsigned idx)
{
	if (idx < get_count()) return data.get_item(idx)->handle_get_track();
	else return 0;
}

void snapshot_history::format_title(unsigned index, string_base & out, const char * spec, const char * p_extra_items)
{
	if (index < get_count())
	{
		history_handle * handle = data.get_item(index);

		extra_items_helper extra_items(p_extra_items);
		int offset;
		if ((int)index < bound_prev) offset = index - bound_prev;
		else if ((int)index > bound_next) offset = index - bound_next;
		else offset = 0;
		if (offset < 0)
		{
			extra_items.add_item("history_prev", -offset);
		}
		else if (offset > 0)
		{
			extra_items.add_item("history_next", offset);
		}
		else if ((int)index == now_playing)
		{
			extra_items.add_item("history_is_playing", "1");
		}
		extra_items.add_item("history_number", index+1);

		handle->handle_format_title(out, spec, extra_items);
	}
	else
	{
		out.reset();
	}
}

bool snapshot_history::is_ordered()
{
	return true;
}

//////////////////////////////////////////////////
// domain
//////////////////////////////////////////////////

// {439753EA-346C-46bb-B013-441DE853B2A6}
const GUID domain_history::domain_guid = 
{ 0x439753ea, 0x346c, 0x46bb, { 0xb0, 0x13, 0x44, 0x1d, 0xe8, 0x53, 0xb2, 0xa6 } };

const char * domain_history::get_name()
{
	return "History";
}

dbsearch::snapshot * domain_history::get_snapshot()
{
	return new service_impl_t< snapshot_history >;
}

bool domain_history::is_ordered()
{
	return false;
}

static dbsearch::domain_factory< domain_history > foo;

#if 0
const char * history_data_source::get_desc()
{
	static const char * text =
		"special fields:\n"
		"- %_history_current% (currently played track only)\n"
		"- %_history_forward% (tracks newer than the currently played only):\n"
		"  how many times you have to go forward to reach this track\n"
		"- %_history_backward% (tracks older than the currently played only):\n"
		"  how many times you have to go back to reach this track\n"
		"- %_history_number%\n"
		"- %_history_timestamp%:\n"
		"  the time at which the entry was created in Universal Time.\n"
		;
	return text;
}
#endif
