#include "../SDK/foobar2000.h"
#include "../helpers/helpers.h"
#include "../foosion_helpers/helpers.h"
#include <history.h>
#include <ui_extension.h>
#include <imp_helper.h>

#include "config.h"
#include "resource.h"

class history_callback_toolbar;

class toolbar_window {
private:
	static const char * CLASSNAME;
	static bool registered;

	static void register_class();

	static LRESULT WINAPI WindowProc(HWND wnd, UINT msg, WPARAM wp, LPARAM lp);

	void show_history_popup(int x1, int x2, int y, int direction);
	bool show_history_popup(int id);

	const char * get_info_tip(int id);

	history_callback_toolbar * callback;

protected:
	const HWND wnd;
	HWND wnd_toolbar;
	ui_extension_host * host;

	toolbar_window(HWND p_wnd);

	LRESULT on_message(UINT msg, WPARAM wp, LPARAM lp);
	LRESULT on_create(WPARAM wp, LPARAM lp);
	LRESULT on_destroy(WPARAM wp, LPARAM lp);
	LRESULT on_size(WPARAM wp, LPARAM lp);
	LRESULT on_command(WPARAM wp, LPARAM lp);
	LRESULT on_notify(WPARAM wp, LPARAM lp);
	LRESULT on_getminmaxinfo(WPARAM wp, LPARAM lp);
	LRESULT on_menuselect(WPARAM wp, LPARAM lp);
	LRESULT on_erasebackground(WPARAM wp, LPARAM lp);
	LRESULT on_windowposchanging(WPARAM wp, LPARAM lp);

public:
	static HWND create(HWND parent, service_base * p_host);

	void check_button_state();
};

class history_callback_toolbar : public history_callback {
private:
	toolbar_window * wnd;

public:
	virtual void on_items_added(int start, int count) {
		wnd->check_button_state();
	}

	virtual void on_items_removing(const bit_array & mask) {}

	virtual void on_items_removed(const bit_array & mask) {
		wnd->check_button_state();
	}

	virtual void on_item_modified(int index) {}

	virtual void on_item_replaced(int index) {}

	virtual void on_playing_track_changed(int from, int to) {
		wnd->check_button_state();
	}

	history_callback_toolbar(toolbar_window * p_wnd) : wnd(p_wnd) {}
};

const char * toolbar_window::CLASSNAME = "{AE1A15E0-EC0D-4b94-BC97-CF7603B8BDFC}";

bool toolbar_window::registered = false;

void toolbar_window::register_class() {
	uWNDCLASS wc;

	wc.style = 0;
	wc.lpfnWndProc = WindowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = sizeof(toolbar_window *);
	wc.hInstance = core_api::get_my_instance();
	wc.hIcon = uLoadIcon(NULL, IDI_APPLICATION);
	wc.hCursor = NULL;
	wc.hbrBackground = NULL;
	wc.lpszClassName = CLASSNAME;
	wc.lpszMenuName = NULL;

	registered = !!uRegisterClass(&wc);
}

HWND toolbar_window::create(HWND parent, service_base * p_host) {
	if (!registered) register_class();
	if (!registered) return NULL;

	return uCreateWindowEx(WS_EX_CONTROLPARENT,
		CLASSNAME, "history buttons",
		WS_CHILD|WS_CLIPCHILDREN|WS_CLIPSIBLINGS,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 
		parent, NULL, core_api::get_my_instance(), p_host);
}

toolbar_window::toolbar_window(HWND p_wnd) : wnd(p_wnd) {
	wnd_toolbar = NULL;
	callback = NULL;
}

LRESULT toolbar_window::WindowProc(HWND wnd, UINT msg, WPARAM wp, LPARAM lp) {
	toolbar_window * _this;
	LRESULT rv;
	if (msg == WM_NCCREATE)
	{
		_this = new toolbar_window(wnd);
		uSetWindowLong(wnd, 0, (long)_this);
	}
	else _this = reinterpret_cast<toolbar_window *>(uGetWindowLong(wnd, 0));

	rv = _this ? _this->on_message(msg, wp, lp) : uDefWindowProc(wnd, msg, wp, lp);

	if (msg == WM_NCDESTROY && _this)
	{
		uSetWindowLong(wnd, 0, 0);
		delete _this;
	}

	return rv;
}

LRESULT toolbar_window::on_message(UINT msg, WPARAM wp, LPARAM lp) {
	switch (msg) {

	case WM_CREATE:
		return on_create(wp, lp);

	case WM_DESTROY:
		return on_destroy(wp, lp);

	case WM_SIZE:
		return on_size(wp, lp);

	case WM_GETMINMAXINFO:
		return on_getminmaxinfo(wp, lp);

	case WM_COMMAND:
		return on_command(wp, lp);

	case WM_NOTIFY:
		return on_notify(wp, lp);

	case WM_MENUSELECT:
		return on_menuselect(wp, lp);

	case WM_ERASEBKGND:
		return on_erasebackground(wp, lp);

	case WM_WINDOWPOSCHANGING:
		return on_windowposchanging(wp, lp);

	}
	return uDefWindowProc(wnd, msg, wp, lp);
}

static HIMAGELIST create_imagelist(UINT id) {
	HIMAGELIST himl = ImageList_Create(15, 14, ILC_COLOR|ILC_MASK, 2, 0);

	HBITMAP bmp = (HBITMAP)uLoadImage(core_api::get_my_instance(), 
		MAKEINTRESOURCE(id), IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR);

	ImageList_AddMasked(himl, bmp, RGB(255, 0, 255));

	DeleteObject(bmp);

	return himl;
}

LRESULT toolbar_window::on_create(WPARAM wp, LPARAM lp) {
	CREATESTRUCT * pcs = (CREATESTRUCT *)lp;

	host = pcs->lpCreateParams ? service_query_t(ui_extension_host, ((service_base *)(pcs->lpCreateParams))) : NULL;

	TBBUTTON tbb[2];

	tbb[0].iBitmap = 0;
	tbb[0].idCommand = ID_HISTORY_BACK;
	tbb[0].fsState = TBSTATE_ENABLED;
	tbb[0].fsStyle = TBSTYLE_DROPDOWN;
	tbb[0].dwData = 0;
	tbb[0].iString = 0;

	tbb[1].iBitmap = 1;
	tbb[1].idCommand = ID_HISTORY_FORWARD;
	tbb[1].fsState = TBSTATE_ENABLED;
	tbb[1].fsStyle = TBSTYLE_DROPDOWN;
	tbb[1].dwData = 0;
	tbb[1].iString = 1;

	const DWORD style = WS_CHILD|WS_VISIBLE|WS_CLIPSIBLINGS|WS_CLIPCHILDREN
		|TBSTYLE_FLAT|TBSTYLE_TRANSPARENT|TBSTYLE_TOOLTIPS
		|CCS_TOP|CCS_NORESIZE|CCS_NOPARENTALIGN|CCS_NODIVIDER;
	wnd_toolbar = CreateToolbarEx(wnd, style, IDC_TOOLBAR,
		2, core_api::get_my_instance(), IDR_TOOLBAR, tbb,
		2, 15, 14, 15, 14, sizeof(TBBUTTON));

	uSendMessage(wnd_toolbar, TB_SETEXTENDEDSTYLE, 0, 
		uSendMessage(wnd_toolbar, TB_GETEXTENDEDSTYLE, 0, 0) | TBSTYLE_EX_DRAWDDARROWS);

	uSendMessage(wnd_toolbar, TB_SETIMAGELIST, 0, (LPARAM)create_imagelist(IDR_TOOLBAR));

	RECT rc;
	GetClientRect(wnd, &rc);
	SetWindowPos(wnd_toolbar, NULL, 0, 0, rc.right, rc.bottom, SWP_NOZORDER);

	callback = new history_callback_toolbar(this);
	history::get()->add_callback(callback);

	check_button_state();

	return 0;
}

LRESULT toolbar_window::on_destroy(WPARAM wp, LPARAM lp) {
	if (host != NULL) host->service_release();

	history::get()->remove_callback(callback);
	delete callback;

	return 0;
}

LRESULT toolbar_window::on_size(WPARAM wp, LPARAM lp) {
	if (wp == SIZE_MAXIMIZED || wp == SIZE_RESTORED) {
		RECT rc;
		GetClientRect(wnd, &rc);
		SetWindowPos(wnd_toolbar, NULL, 0, 0, rc.right, rc.bottom, SWP_NOZORDER);
	}

	return 0;
}

LRESULT toolbar_window::on_getminmaxinfo(WPARAM wp, LPARAM lp) {
	MINMAXINFO * mmi = (MINMAXINFO *)lp;

	SIZE size;
	size.cx = mmi->ptMinTrackSize.x;
	size.cy = mmi->ptMinTrackSize.y;

	if (uSendMessage(wnd_toolbar, TB_GETMAXSIZE, 0, (LPARAM)&size)) {
		mmi->ptMinTrackSize.x = size.cx;
		mmi->ptMinTrackSize.y = size.cy;
		mmi->ptMaxTrackSize.y = size.cy;
	}

	return 0;
}

LRESULT toolbar_window::on_command(WPARAM wp, LPARAM lp) {
	switch (wp) {

	case ID_HISTORY_BACK:
		{
			history::get()->play_item_by_offset(-1);
		}
		break;

	case ID_HISTORY_FORWARD:
		{
			history::get()->play_item_by_offset(+1);
		}
		break;

	}

	return 0;
}

LRESULT toolbar_window::on_notify(WPARAM wp, LPARAM lp) {
	switch (((NMHDR *)lp)->idFrom) {

	case IDC_TOOLBAR:
		{
			switch (((NMHDR *)lp)->code) {

			case TBN_DROPDOWN:
				{
					NMTOOLBAR * nmtb = (NMTOOLBAR *)lp;

					int id = nmtb->iItem;

					return show_history_popup(id) ? TBDDRET_DEFAULT : TBDDRET_NODEFAULT;
				}
				break;

			case NM_RCLICK:
				{
					int id = -1;
#if 0
					// XXX: only version 4.71 and newer
					NMMOUSE * nmm = (NMMOUSE *)lp;

					id = nmm->dwItemSpec;
#else
					POINT pt;
					GetCursorPos(&pt);
					ScreenToClient(wnd_toolbar, &pt);
					int rv = uSendMessage(wnd_toolbar, TB_HITTEST, 0, (LPARAM)&pt);
					if (rv == 0) {
						id = ID_HISTORY_BACK;
					} else if (rv == 1) {
						id = ID_HISTORY_FORWARD;
					}
#endif

					if (id != -1) {
						if (uSendMessage(wnd_toolbar, TB_ISBUTTONENABLED, id, 0)) {
							show_history_popup(id);
						}
						return TRUE;
					}

					return FALSE;
				}
				break;

			case TBN_GETINFOTIPA:
				{
					NMTBGETINFOTIPA * nmtb = (NMTBGETINFOTIPA *)lp;

					const char * tip = get_info_tip(nmtb->iItem);

					string_ansi_from_utf8 tip_os(tip);

					strncpy(nmtb->pszText, tip_os, nmtb->cchTextMax);
				}
				break;

			case TBN_GETINFOTIPW:
				{
					NMTBGETINFOTIPW * nmtb = (NMTBGETINFOTIPW *)lp;

					const char * tip = get_info_tip(nmtb->iItem);

					string_utf16_from_utf8 tip_os(tip);

					wcsncpy(nmtb->pszText, tip_os, nmtb->cchTextMax);
				}
				break;

			}
		}

	}

	return 0;
}

void toolbar_window::show_history_popup(int x1, int x2, int y, int direction) {
	history * h = history::get();

	HMENU menu = CreatePopupMenu();

	bool non_empty = false;

	string8 spec;
	cfg_titleformat_popup.get_value(spec);
	const int max = cfg_max_popup_size;
	for (int n = 1; n <= max; n++) {
		int idx = h->get_index_from_offset(direction*n);
		if (idx == -1) break;
		string8 temp;
		h->format_title(idx, temp, spec, NULL);
		uAppendMenu(menu, MF_STRING, direction*n, temp);
		non_empty = true;
	}

	if (non_empty) {
		UINT flags = TPM_RETURNCMD|TPM_RIGHTBUTTON|TPM_TOPALIGN;
		BOOL right_aligned = FALSE;
		if (!SystemParametersInfo(SPI_GETMENUDROPALIGNMENT, 0, &right_aligned, 0)) {
			right_aligned = FALSE;
		}
		flags |= right_aligned ? TPM_RIGHTALIGN : TPM_LEFTALIGN;
		const int x = right_aligned ? x2 : x1;
		int rv = TrackPopupMenu(menu, flags, x, y, 0, wnd, NULL);
		if (rv != 0) {
			// history contents might have changed after the menu was built,
			// but we ignore that and simply do a relative movement
			h->play_item_by_offset(rv);
		}
	}

	DestroyMenu(menu);
}

bool toolbar_window::show_history_popup(int id) {
	RECT rc;
	uSendMessage(wnd_toolbar, TB_GETRECT, id, (LPARAM)&rc);
	MapWindowPoints(wnd_toolbar, HWND_DESKTOP, (LPPOINT)&rc, 2);                         					
	
	switch (id) {

	case ID_HISTORY_BACK:
		{
			show_history_popup(rc.left, rc.right, rc.bottom, -1);
		}
		break;

	case ID_HISTORY_FORWARD:
		{
			show_history_popup(rc.left, rc.right, rc.bottom, +1);
		}
		break;

	default:
		return false;
	}

	return true;
}

const char * toolbar_window::get_info_tip(int id) {
	switch (id) {

	case ID_HISTORY_BACK:
		return "Back in history";

	case ID_HISTORY_FORWARD:
		return "Forward in history";

	}

	return "";
}

void toolbar_window::check_button_state() {
	history * h = history::get();

	uPostMessage(wnd_toolbar, TB_ENABLEBUTTON, ID_HISTORY_BACK, 
		MAKELONG(h->get_index_from_offset(-1) != -1, 0));

	uPostMessage(wnd_toolbar, TB_ENABLEBUTTON, ID_HISTORY_FORWARD, 
		MAKELONG(h->get_index_from_offset(+1) != -1, 0));
}

LRESULT toolbar_window::on_menuselect(WPARAM wp, LPARAM lp) {
	if (host) {
		UINT flags = (UINT)HIWORD(wp);

		if (flags == 0xffff) {
			host->restore_status_text();
		} else if ((flags & MF_POPUP) == 0) {
			int id = (int)(short)LOWORD(wp);
			if (id < 0) {
				host->override_status_text(string_printf("Goes back %d track%s in history.", -id, id==-1?"":"s"));
			} else if (id > 0) {
				host->override_status_text(string_printf("Goes forward %d track%s in history.", id, id==1?"":"s"));
			}
		}
	}

	return 0;
}

LRESULT toolbar_window::on_erasebackground(WPARAM wp, LPARAM lp) {
	HDC dc = (HDC)wp;

	HWND wnd_parent = GetParent(wnd);
	POINT pt = {0, 0};
	MapWindowPoints(wnd, wnd_parent, &pt, 1);
	OffsetWindowOrgEx(dc, pt.x, pt.y, 0);
	uSendMessage(wnd_parent, WM_ERASEBKGND,wp, 0);
	SetWindowOrgEx(dc, pt.x, pt.y, 0);

	return TRUE;
}

LRESULT toolbar_window::on_windowposchanging(WPARAM wp, LPARAM lp) {
	RedrawWindow(wnd_toolbar, 0, 0, RDW_ERASE);

	return 0;
}




class toolbar_extension_history : public ui_extension_base_t< ui_extension_flag::TYPE_TOOLBAR, UEM_MULTIPLE > {
public:
	virtual const GUID & get_extension_guid() const {
		// {FCA54897-6BD0-438c-B63E-91E987AEDB51}
		static const GUID guid = 
		{ 0xfca54897, 0x6bd0, 0x438c, { 0xb6, 0x3e, 0x91, 0xe9, 0x87, 0xae, 0xdb, 0x51 } };
		return guid;
	}

	virtual void get_name(string_base & out) const {
		out = "History buttons";
	}

	virtual void get_category(string_base & out) const {
		out = "Toolbars";
	}

	virtual HWND create_window(HWND wnd_parent) {
		return toolbar_window::create(wnd_parent, host);
	}
};

static ui_extension_factory< toolbar_extension_history > foo_toolbar_panel;
