#include "history_dialog.h"

#include "config.h"
#include "version.h"
#include "resource.h"

cfg_struct_t<WINDOWPLACEMENT> cfg_window_placement("window_placement",0);


dialog_history dialog_history::g_instance;


dialog_history::dialog_history() 
{
	wnd_status = NULL;
}
 
void dialog_history::activate()
{
	ShowWindow(wnd, SW_SHOW);
	SetForegroundWindow(wnd);
}

void dialog_history::destroy()
{
	DestroyWindow(wnd);
}

BOOL dialog_history::on_initdialog(WPARAM wp, LPARAM lp) {
	BOOL rv = history_dialog_base::on_initdialog(wp, lp);

	apply_window_placement(cfg_window_placement, wnd);
	modeless_dialog_manager::add(wnd);

	wnd_status = uCreateStatusWindow(WS_CHILD | WS_VISIBLE, "", wnd, IDC_STATUS);

	layout();

	return rv;
}

BOOL dialog_history::on_destroy(WPARAM wp, LPARAM lp) {
	history_dialog_base::on_destroy(wp, lp);

	read_window_placement(cfg_window_placement, wnd);
	modeless_dialog_manager::remove(wnd);

	wnd = NULL;

	return TRUE;
}

BOOL dialog_history::on_command(WPARAM wp, LPARAM lp) {
	BOOL rv = history_dialog_base::on_command(wp, lp);

	if (!rv) {
		if (wp == IDCANCEL) {
			DestroyWindow(wnd);
			rv = TRUE;
		}
	}

	return rv;
}

BOOL dialog_history::on_notify(WPARAM wp, LPARAM lp) {
	switch (((LPNMHDR)lp)->idFrom)
	{
	case IDC_STATUS:
		switch (((LPNMHDR)lp)->code)
		{
		case NM_DBLCLK:
			{
				int index = history::get()->get_now_playing();
				if (index >= 0)
					set_focus(index);
			}
			break;
		}
		break;

	default:
		return history_dialog_base::on_notify(wp, lp);
	}

	return TRUE;
}

void dialog_history::layout() {
	uSendMessage(wnd_status, WM_SIZE, 0, 0);

	RECT r;
	static int info[] = {
		1, IDC_LIST,
		1, IDC_STATUS,
		0, 0,
	};

	GetEffectiveClientRect(wnd, &r, info);
	SetWindowPos(wnd_list, NULL, r.left, r.top, r.right-r.left, r.bottom-r.top, SWP_NOZORDER);
}

void dialog_history::set_status_text(const char * text, bool is_menu) {
	if (is_menu && (text == NULL || *text == '\0')) {
		update_status_bar();
	} else {
		uSetWindowText(wnd_status, text);
	}
}

void dialog_history::set_caption_text(const char *text) {
	uSetWindowText(wnd, text);
}

void dialog_history::g_activate() {
	if (g_instance.wnd != NULL) {
		g_instance.activate();
	} else {
		g_instance.run_modeless(IDD_HISTORY, core_api::get_main_window());
	}
}

void dialog_history::g_destroy() {
	if (g_instance.wnd != NULL) {
		g_instance.destroy();
	}
}

class initquit_ui : public initquit {
public:
	virtual void on_init() {
	}

	virtual void on_quit() {
		dialog_history::g_destroy();
	}

	virtual void on_system_shutdown() {
		// XXX: check if we need different handling here
		on_quit();
	}
};

static initquit_factory< initquit_ui > foo_initquit;
