#if !defined(FOO_HISTORY__HISTORY_WINDOW_H)
#define FOO_HISTORY__HISTORY_WINDOW_H

#include "../SDK/foobar2000.h"
#include "../helpers/helpers.h"
#include "../foosion_helpers/helpers.h"
#include <history.h>
#include <ui_extension.h>

class history_callback_ui;

class history_dialog_base : public dialog_base {
private:
	enum {
		ID_PLAY = 1,
		ID_REMOVE,
		ID_PREFS,
		ID_HELP,
		ID_CUSTOM_BASE,
	};

	static LRESULT WINAPI ListHookProc(HWND wnd, UINT msg, WPARAM wp, LPARAM lp);

	WNDPROC ListProcPtr;

	typedef int history_index_t;
	typedef int ui_index_t;

	int count;

	ui_index_t h2ui(history_index_t idx_history);
	history_index_t ui2h(ui_index_t idx_ui);

	config_var_callback_font * font_notify_view;
	config_var_callback_color * color_notify_text;
	config_var_callback_color * color_notify_bg;
	config_var_callback_color * color_notify_sel_text;
	config_var_callback_color * color_notify_sel_bg;

	history_callback_ui * history_notify;

	string8 spec;
	mem_block_fastalloc<char *> cache;

	menu_manager * p_menu_manager;

	HWND wnd_tooltip;
	RECT rc_tooltip;
	int last_tooltip_index;

	void destroy_tooltip();
	BOOL create_tooltip(const char * text);

	HFONT font_view;

	HBRUSH brush_bg;
	HBRUSH brush_sel_bg;

	void draw_item(int index, HDC dc, RECT rc, bool selected, bool focused);

	HFONT get_font();
	COLORREF get_color(int index);
	HBRUSH get_brush(int index);
	void flush_brushes();

	int get_item_height();

	void set_count(int count);

	LRESULT list_hook(UINT msg, WPARAM wp, LPARAM lp);

	BOOL on_drawitem(WPARAM wp, LPARAM lp);
	BOOL on_measureitem(WPARAM wp, LPARAM lp);

	BOOL on_settingchange(WPARAM wp, LPARAM lp);

	BOOL on_ctlcolorlistbox(WPARAM wp, LPARAM lp);

	//BOOL on_windowposchanging(WPARAM wp, LPARAM lp);
	//BOOL on_windowposchanged(WPARAM wp, LPARAM lp);

protected:
	HWND wnd_list;

	history_dialog_base();

	virtual BOOL on_message(UINT msg, WPARAM wp, LPARAM lp);
	BOOL on_menuselect(WPARAM wp, LPARAM lp);

	virtual BOOL on_initdialog(WPARAM wp, LPARAM lp);
	virtual BOOL on_destroy(WPARAM wp, LPARAM lp);

	virtual BOOL on_size(WPARAM wp, LPARAM lp);

	virtual void layout();

	virtual BOOL on_command(WPARAM wp, LPARAM lp);
	virtual BOOL on_notify(WPARAM wp, LPARAM lp);

	virtual BOOL on_keydown(WPARAM wp, LPARAM lp);
	virtual BOOL on_syskeydown(WPARAM wp, LPARAM lp);

	// focus is _not_ an index into the displayed list, but into the history
	void set_focus(history_index_t index);
	history_index_t get_focus();

	void setup_list();

	void update_status_bar();
	void update_list();
	void update_item(int index);

	void play_item();
	void remove_item();

	bool context_menu(HWND child);

	const char * cache_get(history_index_t index);
	void cache_reset();
	void cache_reset_item(history_index_t index);
	void cache_resize(size_t count);

	virtual void set_status_text(const char * text, bool is_menu) {}
	virtual void set_caption_text(const char * text) {}

public:
	void on_history_items_added(int start, int count);
	void on_history_items_removed(const bit_array & mask);
	void on_history_item_modified(int index);
	void on_history_playing_track_changed(int from, int to);
};


class history_callback_ui : public history_callback {
private:
	history_dialog_base * tgt;

public:
	history_callback_ui(history_dialog_base * p_tgt);

	virtual void on_items_added(int start, int count);

	virtual void on_items_removing(const bit_array & mask);

	virtual void on_items_removed(const bit_array & mask);

	virtual void on_item_modified(int index);

	virtual void on_item_replaced(int index);

	virtual void on_playing_track_changed(int from, int to);
};

#endif /* !defined(FOO_HISTORY__HISTORY_WINDOW_H) */
