#include "../SDK/foobar2000.h"
#include "../helpers/helpers.h"
#include "resource.h"
#include "config.h"
#include "prefs.h"

//#include <uxtheme.h>

extern void show_help();

#define ETDT_DISABLE        0x00000001
#define ETDT_ENABLE         0x00000002
#define ETDT_USETABTEXTURE  0x00000004
#define ETDT_ENABLETAB      (ETDT_ENABLE  | ETDT_USETABTEXTURE)

typedef HRESULT WINAPI EnableThemeDialogTextureProc(HWND hwnd, DWORD dwFlags);

class theme_helper {
private:
	unsigned refcount;
	HINSTANCE mod_uxtheme;
	EnableThemeDialogTextureProc * pEnableThemeDialogTexture;

public:
	theme_helper();

	bool load();
	bool unload();

	LRESULT enable_theme_dialog_texture(HWND wnd, DWORD flags);
};

theme_helper::theme_helper() {
	refcount = 0;
	mod_uxtheme = NULL;
	pEnableThemeDialogTexture = NULL;
}

bool theme_helper::load() {
	refcount++;

	if (refcount > 1) {
		return (pEnableThemeDialogTexture != NULL);
	}

	mod_uxtheme = uLoadLibrary("uxtheme.dll");
	if (mod_uxtheme == NULL) {
		return false;
	}

	pEnableThemeDialogTexture =
		(EnableThemeDialogTextureProc *)GetProcAddress(mod_uxtheme, "EnableThemeDialogTexture");

	return (pEnableThemeDialogTexture != NULL);
}

bool theme_helper::unload() {
	refcount--;

	if (refcount > 0) {
		return false;
	}

	if (pEnableThemeDialogTexture != NULL) {
		FreeLibrary(mod_uxtheme);
		mod_uxtheme = NULL;
		pEnableThemeDialogTexture = NULL;
	}

	return true;
}

LRESULT theme_helper::enable_theme_dialog_texture(HWND wnd, DWORD flags) {
	if (pEnableThemeDialogTexture != NULL) {
		return pEnableThemeDialogTexture(wnd, flags);
	} else {
		// TODO: more appropriate return value
		return S_OK;
	}
}

static theme_helper g_theme_helper;

//////////////////////////////////////////////////
// config page controller
//////////////////////////////////////////////////

static config_page_desc_t config_page_table[] = {
	{"Title formatting", dialog_page_titleformat::create},
	{"Display", dialog_page_display::create},
	{"History", dialog_page_history::create},
};

enum {
	config_page_table_size = tabsize(config_page_table),
};

static cfg_int cfg_last_page("last config page", 0);

HWND dialog_config::create(HWND parent) {
	dialog_config * ptr = new dialog_config();
	return ptr->run_modeless(IDD_CONFIG, parent);
}

dialog_config::dialog_config() {
	wnd_tab = 0;
	wnd_page = 0;
}

BOOL dialog_config::on_message(UINT msg, WPARAM wp, LPARAM lp) {
	switch (msg) {

	case WM_INITDIALOG:
		{
			init_tabs();
			if (cfg_last_page < config_page_table_size) {
				uSendMessage(wnd_tab, TCM_SETCURSEL, cfg_last_page, 0);
			}
			change_tab();
		}
		break;

	case WM_DESTROY:
		{
			g_theme_helper.unload();
			delete this;
		}
		break;

	case WM_COMMAND:
		{
			switch (wp) {

			case IDC_HELP_HISTORY:
				{
					show_help();
				}
				break;
			}
		}
		break;

	case WM_NOTIFY:
		return on_notify((NMHDR *) lp);
		break;

	default:
		{
			return FALSE;
		}
		break;
	}

	return TRUE;
}

BOOL dialog_config::on_notify(NMHDR * pnm) {
	switch (pnm->idFrom) {

	case IDC_TAB:
		switch (pnm->code) {

		case TCN_SELCHANGE:
			{
				change_tab();
			}
			break;
		}
		break;
	}

	return TRUE;
}

void dialog_config::init_tabs() {
	wnd_tab = GetDlgItem(wnd, IDC_TAB);
	g_theme_helper.load();

	unsigned n;

	RECT rc_page;
	HWND wnd_empty = GetDlgItem(wnd, IDC_STATIC_EMPTY);
	GetChildRect(wnd, IDC_STATIC_EMPTY, &rc_page);
	for (n = 0; n < config_page_table_size; n++) {
		uTCITEM tab;
		tab.mask = TCIF_TEXT;
		tab.pszText = const_cast< char * >(config_page_table[n].name);
		uTabCtrl_InsertItem(wnd_tab, n, &tab);

		wnd_page = (config_page_table[n].create != 0) ? config_page_table[n].create(wnd) : NULL;
		if (wnd_page != 0) {
			SetWindowPos(wnd_page, wnd_empty, 
				rc_page.left, rc_page.top, rc_page.right-rc_page.left, rc_page.bottom-rc_page.top,
				SWP_HIDEWINDOW);
			g_theme_helper.enable_theme_dialog_texture(wnd_page, ETDT_ENABLETAB);
		}
		wnd_page_list.add_item(wnd_page);
	}
}

void dialog_config::change_tab() {
	if (wnd_page != 0) {
		ShowWindow(wnd_page, SW_HIDE);
		wnd_page = 0;
	}

	unsigned idx = uSendMessage(wnd_tab, TCM_GETCURSEL, 0, 0);

	if (idx != (unsigned)-1) {
		wnd_page = wnd_page_list[idx];
		cfg_last_page = idx;
	}

	HWND wnd_empty = GetDlgItem(wnd, IDC_STATIC_EMPTY);
	if (wnd_page != 0) {
		ShowWindow(wnd_page, SW_SHOW);
		ShowWindow(wnd_empty, SW_HIDE);
	} else {
		ShowWindow(wnd_page, SW_HIDE);
		ShowWindow(wnd_empty, SW_SHOW);
	}
}

class config_history : public config {
public:
	virtual HWND create(HWND parent) {
		return dialog_config::create(parent);
	}

	virtual const char * get_name() {
		return "History";
	}

	virtual const char * get_parent_name() {
		return "Components";
	}
};

static config_factory< config_history > foo_config;
