#include "../SDK/foobar2000.h"

#include "config.h"
#include "prefs.h"
#include "resource.h"


//////////////////////////////////////////////////
// local configuration
//////////////////////////////////////////////////

struct color_desc_t {
	const char * name;
	config_var_int * var;
	bool used;
};

struct font_desc_t {
	config_var_struct< uLOGFONT > * var;
	bool used;
};

struct element_desc_t {
	const char * name;
	color_desc_t color1;
	color_desc_t color2;
	font_desc_t font;
};

static element_desc_t element_table[] = {
	{"Normal",
		{"Text", &cfg_color_text, true},
		{"Background", &cfg_color_bg, true},
		{&cfg_font_history, true},
	},
	{"Selected",
		{"Text", &cfg_color_sel_text, true},
		{"Background", &cfg_color_sel_bg, true},
		{NULL, false},
	},
	{"Active item",
		{"Frame", &cfg_color_frame, true},
		{"not used", NULL, false},
		{NULL, false},
	},
};

enum {
	element_table_size = tabsize(element_table),
};

static cfg_int cfg_last_element("last element", 0);

HWND dialog_page_display::create(HWND parent) {
	dialog_page_display * ptr = new dialog_page_display();
	return ptr->run_modeless(IDD_CFGPAGE_DISPLAY, parent);
}

dialog_page_display::dialog_page_display() {
	brush_sample1 = NULL;
	brush_sample2 = NULL;
}

BOOL dialog_page_display::on_message(UINT msg, WPARAM wp, LPARAM lp) {
	switch(msg)
	{

	case WM_INITDIALOG:
		{
			// appearance
			{
				int n;
				for (n = 0; n < element_table_size; n++) {
					uSendDlgItemMessageText(wnd, IDC_ELEMENT_LIST, CB_ADDSTRING, 0, element_table[n].name);
				}

				n = 0;
				if (cfg_last_element < element_table_size) {
					n = cfg_last_element;
				}
				uSendDlgItemMessage(wnd, IDC_ELEMENT_LIST, CB_SETCURSEL, n, 0);
				set_element(n);
			}

			// colors
			uButton_SetCheck(wnd, IDC_STANDARDCOLORS, !!cfg_toggle_sys_colors);
			uButton_SetCheck(wnd, IDC_STANDARDFONT, !!cfg_toggle_sys_font);
			uButton_SetCheck(wnd, IDC_STANDARDFRAME, !!cfg_toggle_sys_frame);

			// window/sidebar
			uSendDlgItemMessageText(wnd, IDC_DISPLAY_ORDER, CB_ADDSTRING, 0, "newest first");
			uSendDlgItemMessageText(wnd, IDC_DISPLAY_ORDER, CB_ADDSTRING, 0, "oldest first");
			uSendDlgItemMessage(wnd, IDC_DISPLAY_ORDER, CB_SETCURSEL, cfg_toggle_ascending ? 0 : 1, 0);

			// toolbar
			SetDlgItemInt(wnd, IDC_MAXPOPUPSIZE, cfg_max_popup_size, 0);
			uSendDlgItemMessage(wnd, IDC_MAXPOPUPSIZESPIN, UDM_SETRANGE32, 1, 100);
		}
		break;

	case WM_DESTROY:
		{
			if (brush_sample1 != NULL) DeleteObject(brush_sample1);
			if (brush_sample2 != NULL) DeleteObject(brush_sample2);

			delete this;
		}
		break;

	case WM_COMMAND:
		{
			switch (wp) {

				// appearance

			case IDC_STANDARDCOLORS | (BN_CLICKED << 16):
				{
					cfg_toggle_sys_colors = uButton_GetCheck(wnd, IDC_STANDARDCOLORS);
				}
				break;

			case IDC_STANDARDFONT | (BN_CLICKED << 16):
				{
					cfg_toggle_sys_font = uButton_GetCheck(wnd, IDC_STANDARDFONT);
				}
				break;

			case IDC_STANDARDFRAME | (BN_CLICKED << 16):
				{
					cfg_toggle_sys_frame = uButton_GetCheck(wnd, IDC_STANDARDFRAME);
				}
				break;

			case IDC_ELEMENT_LIST | (CBN_SELCHANGE << 16):
				{
					int n = uSendDlgItemMessage(wnd, IDC_ELEMENT_LIST, CB_GETCURSEL, 0, 0);
					cfg_last_element = n;
					set_element(n);
				}
				break;

				// colors

			case IDC_COLOR1:
				{
					edit_color(0);
				}
				break;

			case IDC_COLOR2:
				{
					edit_color(1);
				}
				break;

				// fonts

			case IDC_FONT:
				{
					edit_font(cfg_font_history);

					string8 desc;
					get_font_description(cfg_font_history, desc);
					uSetDlgItemText(wnd, IDC_FONT_DESC, desc);
				}
				break;

				// window/sidebar

			case IDC_DISPLAY_ORDER | (CBN_SELCHANGE << 16):
				{
					int idx = uSendDlgItemMessage(wnd, IDC_DISPLAY_ORDER, CB_GETCURSEL, 0, 0);
					cfg_toggle_ascending = idx ? 0 : 1;
				}
				break;

				// toolbar

			case IDC_MAXPOPUPSIZE | (EN_KILLFOCUS<<16):
				{
					BOOL translated;
					unsigned max_popup_size = GetDlgItemInt(wnd, IDC_MAXPOPUPSIZE, &translated, 0);
					if (translated) cfg_max_popup_size = max_popup_size;
				}
				break;

			}
		}
		break;

	case WM_CTLCOLORSTATIC:
		{
			if (GetDlgItem(wnd, IDC_COLOR1_SAMPLE) == (HWND)lp) {
				HDC dc =(HDC)wp;
				if (brush_sample1 == NULL) {
					int elt = uSendDlgItemMessage(wnd, IDC_ELEMENT_LIST, CB_GETCURSEL, 0, 0);
					color_desc_t & color = element_table[elt].color1;
					if (color.var != NULL) {
						brush_sample1 = CreateSolidBrush(color.var->get_value());
					}
				}
				return (BOOL)brush_sample1;
			} else if (GetDlgItem(wnd, IDC_COLOR2_SAMPLE) == (HWND)lp) {
				HDC dc =(HDC)wp;
				if (brush_sample2 == NULL) {
					int elt = uSendDlgItemMessage(wnd, IDC_ELEMENT_LIST, CB_GETCURSEL, 0, 0);
					color_desc_t & color = element_table[elt].color2;
					if (color.var != NULL) {
						brush_sample2 = CreateSolidBrush(color.var->get_value());
					}
				}
				return (BOOL)brush_sample2;
			}

			return FALSE;
		}
		break;

	default:
		return FALSE;
		break;
	}

	return TRUE;
}

bool dialog_page_display::edit_color(int n) {
	bool rv = false;

	assert(n == 0 || n == 1);

	int elt = uSendDlgItemMessage(wnd, IDC_ELEMENT_LIST, CB_GETCURSEL, 0, 0);

	color_desc_t & color = (n == 0) ? element_table[elt].color1 : element_table[elt].color2;

	assert(color.used && color.var != NULL);

	config_var_int * var = color.var;

	color_table t = cfg_color_table;
	DWORD val = var->get_value();
	if (uChooseColor(&val, wnd, t.color)) {
		var->set_value(val);
		cfg_color_table = t;

		HBRUSH & brush = (n == 0) ? brush_sample1 : brush_sample2;
		if (brush != NULL) {
			DeleteObject(brush);
			brush = NULL;
		}
		RedrawWindow(GetDlgItem(wnd, (n == 0) ? IDC_COLOR1_SAMPLE : IDC_COLOR2_SAMPLE), NULL, NULL, RDW_INVALIDATE);

		rv = true;
	}
	return rv;
}

bool dialog_page_display::edit_font(config_var_struct< uLOGFONT > & var) {
	bool rv = false;
	uLOGFONT font;
	var.get_value(font);
	if (uChooseFont(&font, wnd)) {
		var.set_value(font);
		rv = true;
	}
	return rv;
}

void dialog_page_display::get_font_description(const uLOGFONT & font, string_base & out) {
	const char * facename = font.lfFaceName;
	//lfHeight = -MulDiv(PointSize, GetDeviceCaps(hDC, LOGPIXELSY), 72);
	HDC dc = GetDC(NULL);
	int pt = -MulDiv(font.lfHeight, 72, GetDeviceCaps(dc, LOGPIXELSY));
	ReleaseDC(NULL, dc);
	bool bold = (font.lfWeight > FW_NORMAL);
	bool italic = !!font.lfItalic;

	out = string_printf("%s%s%s %dpt", facename, bold ? " bold" : "", italic ? " italic" : "", pt);
}

void dialog_page_display::set_element(int n) {
	assert(n >= 0);
	assert(n < element_table_size);

	element_desc_t & elt = element_table[n];

	// color 1
	if (elt.color1.used) {
		assert(elt.color1.var != NULL);

		EnableWindow(GetDlgItem(wnd, IDC_COLOR1), TRUE);
		EnableWindow(GetDlgItem(wnd, IDC_COLOR1_DESC), TRUE);
	} else {
		EnableWindow(GetDlgItem(wnd, IDC_COLOR1), FALSE);
		EnableWindow(GetDlgItem(wnd, IDC_COLOR1_DESC), FALSE);
	}

	uSetDlgItemText(wnd, IDC_COLOR1_DESC, elt.color1.name);

	if (brush_sample1 != NULL) {
		DeleteObject(brush_sample1);
		brush_sample1 = NULL;
	}
	RedrawWindow(GetDlgItem(wnd, IDC_COLOR1_SAMPLE), NULL, NULL, RDW_INVALIDATE);

	// color 2
	if (elt.color2.used) {
		assert(elt.color2.var != NULL);

		EnableWindow(GetDlgItem(wnd, IDC_COLOR2), TRUE);
		EnableWindow(GetDlgItem(wnd, IDC_COLOR2_DESC), TRUE);
	} else {
		EnableWindow(GetDlgItem(wnd, IDC_COLOR2), FALSE);
		EnableWindow(GetDlgItem(wnd, IDC_COLOR2_DESC), FALSE);
	}

	uSetDlgItemText(wnd, IDC_COLOR2_DESC, elt.color2.name);

	if (brush_sample2 != NULL) {
		DeleteObject(brush_sample2);
		brush_sample2 = NULL;
	}
	RedrawWindow(GetDlgItem(wnd, IDC_COLOR2_SAMPLE), NULL, NULL, RDW_INVALIDATE);


	// font
	if (elt.font.used) {
		assert(elt.font.var != NULL);

		EnableWindow(GetDlgItem(wnd, IDC_FONT), TRUE);
		EnableWindow(GetDlgItem(wnd, IDC_FONT_DESC), TRUE);

		uLOGFONT font;
		elt.font.var->get_value(font);

		string8 desc;
		get_font_description(font, desc);

		uSetDlgItemText(wnd, IDC_FONT_DESC, desc);
	} else {
		EnableWindow(GetDlgItem(wnd, IDC_FONT), FALSE);
		EnableWindow(GetDlgItem(wnd, IDC_FONT_DESC), FALSE);

		uSetDlgItemText(wnd, IDC_FONT_DESC, "");
	}
}
