#include "../SDK/foobar2000.h"
#include "../foosion_helpers/helpers.h"
#include "config.h"
#include "prefs.h"
#include "resource.h"

enum {
	FORMAT_SUPPORTS_COLORS = 1,
	FORMAT_SUPPORTS_TABS = 2,

	FORMAT_SUPPORTS_COLORS_TABS = FORMAT_SUPPORTS_COLORS | FORMAT_SUPPORTS_TABS,
};

struct titleformat_desc_t {
	const char * name;
	config_var_string * var;
	unsigned flags;
};

static titleformat_desc_t titleformat_table[] = {
	{"History", &cfg_titleformat_history, FORMAT_SUPPORTS_COLORS_TABS},
	{"Status bar", &cfg_titleformat_statusbar, 0},
	{"Toolbar popup", &cfg_titleformat_popup, 0},
};

enum {
	titleformat_table_size = tabsize(titleformat_table),
};

static cfg_int cfg_last_titleformat("last titleformat", 0);

static cfg_int cfg_last_color("last color", RGB(255, 255, 255));



static history_handle * get_handle() {
	history_handle * rv = NULL;
	history * ptr = history::get();
	if (ptr != NULL) {
		int idx = ptr->get_now_playing();
		if (idx == -1) idx = 0;
		rv = ptr->get_item(idx);
	}
	return rv;
}

static void update_preview(HWND wnd, history_handle * handle, const char *spec) {
	if (handle != NULL) {
		static const char * extra_items = "history_number=1\0";
		string8 preview;
		handle->handle_format_title(preview, spec, extra_items);
		uSetWindowText(wnd, preview);
	}
}




HWND dialog_page_titleformat::create(HWND parent) {
	dialog_page_titleformat * ptr = new dialog_page_titleformat();
	return ptr->run_modeless(IDD_CFGPAGE_TITLEFORMAT, parent);
}

dialog_page_titleformat::dialog_page_titleformat() {
}

BOOL dialog_page_titleformat::on_message(UINT msg, WPARAM wp, LPARAM lp) {
	switch(msg)
	{

	case WM_INITDIALOG:
		handle = get_handle();

		// titleformatting
		{
			current_format = NULL;

			int n;
			for (n = 0; n < titleformat_table_size; n++) {
				uSendDlgItemMessageText(wnd, IDC_FORMAT_LIST, CB_ADDSTRING, 0, titleformat_table[n].name);
			}
			int current_tab = (cfg_last_titleformat > titleformat_table_size) ? 0 : cfg_last_titleformat;
			uSendDlgItemMessage(wnd, IDC_FORMAT_LIST, CB_SETCURSEL, current_tab, 0);

			string8 spec;
			titleformat_table[current_tab].var->get_value(spec);
			uSetDlgItemText(wnd, IDC_FORMAT, spec);
			current_format = titleformat_table[current_tab].var;
			update_preview(uGetDlgItem(wnd, IDC_PREVIEW), handle, spec);
		}
		break;

	case WM_DESTROY:
		{
			if (handle) handle->handle_release();
			
			delete this;
		}
		break;

	case WM_KILLFOCUS:
		{
		}
		break;

	case WM_COMMAND:
		{
			switch (wp) {

			case IDC_RESET_FORMAT:
				{
					current_format->reset();
					string8 spec;
					current_format->get_value(spec);
					uSetDlgItemText(wnd, IDC_FORMAT, spec);
					update_preview(uGetDlgItem(wnd, IDC_PREVIEW), handle, spec);
				}
				break;

			case IDC_FORMAT | (EN_KILLFOCUS<<16):
				save_text();
				break;

			case IDC_FORMAT | (EN_CHANGE<<16):
				{
					on_text_changed();
				}
				break;

			case IDC_FORMAT_LIST | (CBN_SELCHANGE << 16):
				{
					if (current_format) current_format->set_value(string_utf8_from_window(wnd, IDC_FORMAT));
					int n = uSendDlgItemMessage(wnd, IDC_FORMAT_LIST, CB_GETCURSEL, 0, 0);
					cfg_last_titleformat = n;
					current_format = titleformat_table[n].var;
					string8 spec;
					current_format->get_value(spec);
					uSetDlgItemText(wnd, IDC_FORMAT, spec);
					update_preview(uGetDlgItem(wnd, IDC_PREVIEW), handle, spec);
				}
				break;

			case IDC_INSERT_FORMAT:
				{
					show_insert_format_menu();
				}
				break;

			}
		}
		break;

	default:
		return FALSE;
		break;
	}

	return TRUE;
}

void dialog_page_titleformat::on_text_changed() {
	string_utf8_from_window spec(wnd, IDC_FORMAT);
	update_preview(uGetDlgItem(wnd, IDC_PREVIEW), handle, spec);
}

void dialog_page_titleformat::save_text() {
	current_format->set_value(string_utf8_from_window(wnd, IDC_FORMAT));
}

struct field_desc_t {
	const char * name; // NULL for separator
	const char * val;
};

#define SEPARATOR {NULL, NULL}
struct field_desc_t field_table[] = {
	{"Position", "%_history_number%"},
	{"Relative position", "$if3(-%_history_prev%,+%_history_next%,$if(%_history_is_playing%,0))"},
	SEPARATOR,
	{"Date", "%_history_date%"},
	{"Time", "%_history_time%"},
	{"Day of week", "$select(%_history_day_of_week%,Sunday,Monday,Tuesday,Wednesday,Thursday,Friday,Saturday)"},
	{"Day of week (short name)", "$select(%_history_day_of_week%,Sun,Mon,Tue,Wed,Thu,Fri,Sat)"},
	{"Day of week (index)", "%_history_day_of_week%"},
	{"Days ago", "%_history_days_ago%"},
	{"Hours ago", "%_history_hours_ago%"},
	{"Timezone", "%_history_timezone%"},
	//{"Timezone bias in minutes", "%_history_timezone_minutes%"},
	SEPARATOR,
	{"Timestamp (GMT)", "%_history_timestamp%"},
};
#undef SEPARATOR

enum {
	field_table_size = tabsize(field_table),
};

enum {
	ID_INSERT_COLOR = 1,
	ID_INSERT_TAB,
	ID_CUSTOM_BASE,
};

void dialog_page_titleformat::show_insert_format_menu() {
	RECT rc;
	GetWindowRect(GetDlgItem(wnd, IDC_INSERT_FORMAT), &rc);

	HMENU menu = CreatePopupMenu();

	int sel = uSendDlgItemMessage(wnd, IDC_FORMAT_LIST, CB_GETCURSEL, 0, 0);

	unsigned format_flags = 0;

	if (sel >= 0 && sel < titleformat_table_size) {
		format_flags = titleformat_table[sel].flags;
	}

	bool insert_separator = false;

	if (format_flags & FORMAT_SUPPORTS_COLORS) {
		uAppendMenu(menu, MF_STRING, ID_INSERT_COLOR, "Color...");
		insert_separator = true;
	}

	if (format_flags & FORMAT_SUPPORTS_TABS) {
		uAppendMenu(menu, MF_STRING, ID_INSERT_TAB, "Tab");
		insert_separator = true;
	}

	int n;
	for (n = 0; n < field_table_size; n++) {
		const char * name = field_table[n].name;
		if (name != NULL) {
			if (insert_separator) {
				uAppendMenu(menu, MF_SEPARATOR, 0, NULL);
				insert_separator = false;
			}

			uAppendMenu(menu, MF_STRING, n+ID_CUSTOM_BASE, name);
		} else {
			insert_separator = true;
		}
	}

	menu_manager * p_menu_manager = menu_manager::create();

	if (p_menu_manager != NULL) {
		p_menu_manager->v_win32_auto_mnemonics(menu);

		p_menu_manager->service_release();
	}

	UINT flags = TPM_RETURNCMD|TPM_NONOTIFY|TPM_TOPALIGN;
	BOOL right_aligned = FALSE;
	if (!SystemParametersInfo(SPI_GETMENUDROPALIGNMENT, 0, &right_aligned, 0)) {
		right_aligned = FALSE;
	}
	flags |= right_aligned ? TPM_RIGHTALIGN : TPM_LEFTALIGN;
	int rv = TrackPopupMenu(menu, flags, right_aligned ? rc.right : rc.left, rc.bottom, 0, wnd, NULL);

	if (rv == ID_INSERT_COLOR) {
		color_table t = cfg_color_table;
		COLORREF color = cfg_last_color;
		if (uChooseColor(&color, wnd, t.color)) {
			cfg_last_color = color;
			cfg_color_table = t;
			uSendDlgItemMessageText(wnd, IDC_FORMAT, EM_REPLACESEL, TRUE,
				string_printf("$rgb(%d,%d,%d)", GetRValue(color), GetGValue(color), GetBValue(color)));
		}
	} else if (rv == ID_INSERT_TAB) {
		uSendDlgItemMessageText(wnd, IDC_FORMAT, EM_REPLACESEL, TRUE, "$tab()");
	} else if (rv >= ID_CUSTOM_BASE) {
		const char * val = field_table[rv - ID_CUSTOM_BASE].val;
		if (val != NULL) {
			uSendDlgItemMessageText(wnd, IDC_FORMAT, EM_REPLACESEL, TRUE, val);
		}
	}

	DestroyMenu(menu);

	on_text_changed();
	save_text();
}
